@@ -25,6 +25,7 @@ import jinja2
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
+from oslo_utils import fileutils
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
@@ -349,7 +350,8 @@ def create_pxe_config(task, pxe_options, template=None, ipxe_enabled=False):
               'DISK_IDENTIFIER': pxe_config_disk_ident}
 
     pxe_config = utils.render_template(template, params)
-    utils.write_to_file(pxe_config_file_path, pxe_config)
+    utils.write_to_file(pxe_config_file_path, pxe_config,
+                        CONF.pxe.file_permission)
 
     # Always write the mac addresses
     _link_mac_pxe_configs(task, ipxe_enabled=ipxe_enabled)
@@ -382,7 +384,8 @@ def create_ipxe_boot_script():
     # which should be rather rare
     if (not os.path.isfile(bootfile_path)
             or not utils.file_has_content(bootfile_path, boot_script)):
-        utils.write_to_file(bootfile_path, boot_script)
+        utils.write_to_file(bootfile_path, boot_script,
+                            CONF.pxe.file_permission)
 
 
 def clean_up_pxe_config(task, ipxe_enabled=False):
@@ -1195,7 +1198,8 @@ def prepare_instance_kickstart_config(task, image_info, anaconda_boot=False):
     ks_options = build_kickstart_config_options(task)
     kickstart_template = image_info['ks_template'][1]
     ks_cfg = utils.render_template(kickstart_template, ks_options)
-    utils.write_to_file(image_info['ks_cfg'][1], ks_cfg)
+    utils.write_to_file(image_info['ks_cfg'][1], ks_cfg,
+                        CONF.pxe.file_permission)
 
 
 @image_cache.cleanup(priority=25)
@@ -1234,8 +1238,12 @@ def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
 
     LOG.debug("Fetching necessary kernel and ramdisk for node %s",
               node.uuid)
-    deploy_utils.fetch_images(ctx, TFTPImageCache(), list(t_pxe_info.values()),
+    images_info = list(t_pxe_info.values())
+    deploy_utils.fetch_images(ctx, TFTPImageCache(), images_info,
                               CONF.force_raw_images)
+    if CONF.pxe.file_permission:
+        for info in images_info:
+            os.chmod(info[1], CONF.pxe.file_permission)
 
 
 def clean_up_pxe_env(task, images_info, ipxe_enabled=False):
@@ -1303,3 +1311,25 @@ def place_loaders_for_boot(base_path):
                    'the requested destination. %s' % e)
             LOG.error(msg)
             raise exception.IncorrectConfiguration(error=msg)
+
+
+def place_common_config():
+    """Place template generated config which is not node specific.
+
+    Currently places the initial grub config for grub network boot.
+    """
+    if not CONF.pxe.initial_grub_template:
+        return
+
+    grub_dir_path = os.path.join(_get_root_dir(False), 'grub')
+    if not os.path.isdir(grub_dir_path):
+        fileutils.ensure_tree(grub_dir_path)
+        if CONF.pxe.dir_permission:
+            os.chmod(grub_dir_path, CONF.pxe.dir_permission)
+
+    initial_grub = utils.render_template(
+        CONF.pxe.initial_grub_template,
+        {'tftp_root': _get_root_dir(False)})
+    initial_grub_path = os.path.join(grub_dir_path, 'grub.cfg')
+
+    utils.write_to_file(initial_grub_path, initial_grub)
