@@ -33,12 +33,15 @@ from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common import faults
 from ironic.common.i18n import _
+from ironic.common import images
 from ironic.common import network
 from ironic.common import nova
 from ironic.common import states
+from ironic.common import utils
 from ironic.conductor import notification_utils as notify_utils
 from ironic.conductor import task_manager
 from ironic.objects import fields
+from ironic.objects import node_history
 
 LOG = log.getLogger(__name__)
 CONF = cfg.CONF
@@ -133,14 +136,22 @@ def node_set_boot_mode(task, mode):
         return
 
     task.driver.management.validate(task)
+    try:
+        supported_boot_modes = (
+            task.driver.management.get_supported_boot_modes(task)
+        )
+    except exception.UnsupportedDriverExtension:
+        LOG.debug(
+            "Cannot determine supported boot modes of driver "
+            "%(driver)s. Will make an attempt to set boot mode %(mode)s",
+            {'driver': task.node.driver, 'mode': mode})
+        supported_boot_modes = ()
 
-    boot_modes = task.driver.management.get_supported_boot_modes(task)
-
-    if mode not in boot_modes:
+    if supported_boot_modes and mode not in supported_boot_modes:
         msg = _("Unsupported boot mode %(mode)s specified for "
                 "node %(node_id)s. Supported boot modes are: "
                 "%(modes)s") % {'mode': mode,
-                                'modes': ', '.join(boot_modes),
+                                'modes': ', '.join(supported_boot_modes),
                                 'node_id': task.node.uuid}
         raise exception.InvalidParameterValue(msg)
 
@@ -229,17 +240,18 @@ def _can_skip_state_change(task, new_state):
         notify_utils.emit_power_set_notification(
             task, fields.NotificationLevel.INFO,
             fields.NotificationStatus.END, new_state)
-        LOG.warning("Not going to change node %(node)s power state because "
-                    "current state = requested state = '%(state)s'.",
-                    {'node': node.uuid, 'state': curr_state})
+        LOG.debug("Not going to change node %(node)s power state because "
+                  "current state = requested state = '%(state)s'.",
+                  {'node': node.uuid, 'state': curr_state})
 
     try:
         curr_state = task.driver.power.get_power_state(task)
     except Exception as e:
         with excutils.save_and_reraise_exception():
-            node['last_error'] = _(
+            error = _(
                 "Failed to change power state to '%(target)s'. "
                 "Error: %(error)s") % {'target': new_state, 'error': e}
+            node_history_record(node, event=error, error=True)
             node['target_power_state'] = states.NOSTATE
             node.save()
             notify_utils.emit_power_set_notification(
@@ -293,10 +305,7 @@ def node_power_action(task, new_state, timeout=None):
     # and clients that work is in progress.
     node['target_power_state'] = target_state
     node['last_error'] = None
-    driver_internal_info = node.driver_internal_info
-    driver_internal_info['last_power_state_change'] = str(
-        timeutils.utcnow().isoformat())
-    node.driver_internal_info = driver_internal_info
+    node.timestamp_driver_internal_info('last_power_state_change')
     # NOTE(dtantsur): wipe token on shutting down, otherwise a reboot in
     # fast-track (or an accidentally booted agent) will cause subsequent
     # actions to fail.
@@ -321,12 +330,13 @@ def node_power_action(task, new_state, timeout=None):
     except Exception as e:
         with excutils.save_and_reraise_exception():
             node['target_power_state'] = states.NOSTATE
-            node['last_error'] = _(
+            error = _(
                 "Failed to change power state to '%(target_state)s' "
                 "by '%(new_state)s'. Error: %(error)s") % {
                     'target_state': target_state,
                     'new_state': new_state,
                     'error': e}
+            node_history_record(node, event=error, error=True)
             node.save()
             notify_utils.emit_power_set_notification(
                 task, fields.NotificationLevel.ERROR,
@@ -391,7 +401,9 @@ def provisioning_error_handler(e, node, provision_state,
         #             because it isn't updated on a failed deploy
         node.provision_state = provision_state
         node.target_provision_state = target_provision_state
-        node.last_error = (_("No free conductor workers available"))
+        error = (_("No free conductor workers available"))
+        node_history_record(node, event=error, event_type=states.PROVISIONING,
+                            error=True)
         node.save()
         LOG.warning("No free conductor workers available to perform "
                     "an action on node %(node)s, setting node's "
@@ -462,20 +474,19 @@ def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
             states.CLEANFAIL):
         # Clear clean step, msg should already include current step
         node.clean_step = {}
-        info = node.driver_internal_info
         # Clear any leftover metadata about cleaning
-        info.pop('clean_step_index', None)
-        info.pop('cleaning_reboot', None)
-        info.pop('cleaning_polling', None)
-        info.pop('skip_current_clean_step', None)
+        node.del_driver_internal_info('clean_step_index')
+        node.del_driver_internal_info('cleaning_reboot')
+        node.del_driver_internal_info('cleaning_polling')
+        node.del_driver_internal_info('skip_current_clean_step')
         # We don't need to keep the old agent URL
         # as it should change upon the next cleaning attempt.
-        info.pop('agent_url', None)
-        node.driver_internal_info = info
+        node.del_driver_internal_info('agent_url')
     # For manual cleaning, the target provision state is MANAGEABLE, whereas
     # for automated cleaning, it is AVAILABLE.
     manual_clean = node.target_provision_state == states.MANAGEABLE
-    node.last_error = errmsg
+    node_history_record(node, event=errmsg, event_type=states.CLEANING,
+                        error=True)
     # NOTE(dtantsur): avoid overwriting existing maintenance_reason
     if not node.maintenance_reason and set_maintenance:
         node.maintenance_reason = errmsg
@@ -488,32 +499,29 @@ def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
 
 def wipe_internal_info_on_power_off(node):
     """Wipe information that should not survive reboot/power off."""
-    driver_internal_info = node.driver_internal_info
     # DHCP may result in a new IP next time.
-    driver_internal_info.pop('agent_url', None)
+    node.del_driver_internal_info('agent_url')
     if not is_agent_token_pregenerated(node):
         # Wipe the token if it's not pre-generated, otherwise we'll refuse to
         # generate it again for the newly booted agent.
-        driver_internal_info.pop('agent_secret_token', False)
+        node.del_driver_internal_info('agent_secret_token')
     # Wipe cached steps since they may change after reboot.
-    driver_internal_info.pop('agent_cached_deploy_steps', None)
-    driver_internal_info.pop('agent_cached_clean_steps', None)
+    node.del_driver_internal_info('agent_cached_deploy_steps')
+    node.del_driver_internal_info('agent_cached_clean_steps')
     # Remove TLS certificate since it's regenerated on each run.
-    driver_internal_info.pop('agent_verify_ca', None)
-    node.driver_internal_info = driver_internal_info
+    node.del_driver_internal_info('agent_verify_ca')
 
 
 def wipe_token_and_url(task):
     """Remove agent URL and token from the task."""
-    info = task.node.driver_internal_info
-    info.pop('agent_secret_token', None)
-    info.pop('agent_secret_token_pregenerated', None)
+    node = task.node
+    node.del_driver_internal_info('agent_secret_token')
+    node.del_driver_internal_info('agent_secret_token_pregenerated')
     # Remove agent_url since it will be re-asserted
     # upon the next deployment attempt.
-    info.pop('agent_url', None)
+    node.del_driver_internal_info('agent_url')
     # Remove TLS certificate since it's regenerated on each run.
-    info.pop('agent_verify_ca', None)
-    task.node.driver_internal_info = info
+    node.del_driver_internal_info('agent_verify_ca')
 
 
 def wipe_deploy_internal_info(task):
@@ -521,32 +529,30 @@ def wipe_deploy_internal_info(task):
     if not fast_track_able(task):
         wipe_token_and_url(task)
     # Clear any leftover metadata about deployment.
-    info = task.node.driver_internal_info
-    info['deploy_steps'] = None
-    info.pop('user_deploy_steps', None)
-    info.pop('agent_cached_deploy_steps', None)
-    info.pop('deploy_step_index', None)
-    info.pop('deployment_reboot', None)
-    info.pop('deployment_polling', None)
-    info.pop('skip_current_deploy_step', None)
-    info.pop('steps_validated', None)
-    task.node.driver_internal_info = info
+    node = task.node
+    node.set_driver_internal_info('deploy_steps', None)
+    node.del_driver_internal_info('user_deploy_steps')
+    node.del_driver_internal_info('agent_cached_deploy_steps')
+    node.del_driver_internal_info('deploy_step_index')
+    node.del_driver_internal_info('deployment_reboot')
+    node.del_driver_internal_info('deployment_polling')
+    node.del_driver_internal_info('skip_current_deploy_step')
+    node.del_driver_internal_info('steps_validated')
 
 
 def wipe_cleaning_internal_info(task):
     """Remove temporary cleaning fields from driver_internal_info."""
     if not fast_track_able(task):
         wipe_token_and_url(task)
-    info = task.node.driver_internal_info
-    info['clean_steps'] = None
-    info.pop('agent_cached_clean_steps', None)
-    info.pop('clean_step_index', None)
-    info.pop('cleaning_reboot', None)
-    info.pop('cleaning_polling', None)
-    info.pop('cleaning_disable_ramdisk', None)
-    info.pop('skip_current_clean_step', None)
-    info.pop('steps_validated', None)
-    task.node.driver_internal_info = info
+    node = task.node
+    node.set_driver_internal_info('clean_steps', None)
+    node.del_driver_internal_info('agent_cached_clean_steps')
+    node.del_driver_internal_info('clean_step_index')
+    node.del_driver_internal_info('cleaning_reboot')
+    node.del_driver_internal_info('cleaning_polling')
+    node.del_driver_internal_info('cleaning_disable_ramdisk')
+    node.del_driver_internal_info('skip_current_clean_step')
+    node.del_driver_internal_info('steps_validated')
 
 
 def deploying_error_handler(task, logmsg, errmsg=None, traceback=False,
@@ -562,7 +568,8 @@ def deploying_error_handler(task, logmsg, errmsg=None, traceback=False,
     errmsg = errmsg or logmsg
     node = task.node
     LOG.error(logmsg, exc_info=traceback)
-    node.last_error = errmsg
+    node_history_record(node, event=errmsg, event_type=states.DEPLOYING,
+                        error=True)
     node.save()
 
     cleanup_err = None
@@ -592,7 +599,9 @@ def deploying_error_handler(task, logmsg, errmsg=None, traceback=False,
         wipe_deploy_internal_info(task)
 
     if cleanup_err:
-        node.last_error = cleanup_err
+        node_history_record(node, event=cleanup_err,
+                            event_type=states.DEPLOYING,
+                            error=True)
     node.save()
 
     # NOTE(tenbrae): there is no need to clear conductor_affinity
@@ -614,6 +623,32 @@ def fail_on_error(error_callback, msg, *error_args, **error_kwargs):
     return wrapper
 
 
+def verifying_error_handler(task, logmsg, errmsg=None, traceback=False):
+
+    """Handle errors during verification steps
+
+    :param task: the task
+    :param logmsg: message to be logged
+    :param errmsg: message for the user
+    :param traceback: Boolean; True to log a traceback
+    """
+    errmsg = errmsg or logmsg
+    node = task.node
+    LOG.error(logmsg, exc_info=traceback)
+    node_history_record(node, event=errmsg, event_type=states.VERIFYING,
+                        error=True)
+    node.save()
+
+    node.refresh()
+    if node.provision_state in (
+            states.VERIFYING):
+        # Clear verifying step; we leave the list of verify steps
+        # in node.driver_internal_info for debugging purposes.
+        node.verify_step = {}
+
+    node.save()
+
+
 @task_manager.require_exclusive_lock
 def abort_on_conductor_take_over(task):
     """Set node's state when a task was aborted due to conductor take over.
@@ -628,7 +663,8 @@ def abort_on_conductor_take_over(task):
     else:
         # For aborted deployment (and potentially other operations), just set
         # the last_error accordingly.
-        task.node.last_error = msg
+        node_history_record(task.node, event=msg, event_type=states.TAKEOVER,
+                            error=True)
         task.node.save()
 
     LOG.warning('Aborted the current operation on node %s due to '
@@ -649,17 +685,22 @@ def rescuing_error_handler(task, msg, set_fail_state=True):
         node_power_action(task, states.POWER_OFF)
         task.driver.rescue.clean_up(task)
         remove_agent_url(node)
-        node.last_error = msg
+        node_history_record(task.node, event=msg, event_type=states.RESCUE,
+                            error=True)
     except exception.IronicException as e:
-        node.last_error = (_('Rescue operation was unsuccessful, clean up '
-                             'failed for node: %(error)s') % {'error': e})
+        error = (_('Rescue operation was unsuccessful, clean up '
+                   'failed for node: %(error)s') % {'error': e})
+        node_history_record(task.node, event=error, event_type=states.RESCUE,
+                            error=True)
         LOG.error(('Rescue operation was unsuccessful, clean up failed for '
                    'node %(node)s: %(error)s'),
                   {'node': node.uuid, 'error': e})
     except Exception as e:
-        node.last_error = (_('Rescue failed, but an unhandled exception was '
-                             'encountered while aborting: %(error)s') %
-                           {'error': e})
+        error = (_('Rescue failed, but an unhandled exception was '
+                   'encountered while aborting: %(error)s') %
+                 {'error': e})
+        node_history_record(task.node, event=error, event_type=states.RESCUE,
+                            error=True)
         LOG.exception('Rescue failed for node %(node)s, an exception was '
                       'encountered while aborting.', {'node': node.uuid})
     finally:
@@ -700,7 +741,9 @@ def _spawn_error_handler(e, node, operation):
     :param operation: the operation being performed on the node.
     """
     if isinstance(e, exception.NoFreeConductorWorker):
-        node.last_error = (_("No free conductor workers available"))
+        error = (_("No free conductor workers available"))
+        node_history_record(node, event=error, event_type=states.CONDUCTOR,
+                            error=True)
         node.save()
         LOG.warning("No free conductor workers available to perform "
                     "%(operation)s on node %(node)s",
@@ -741,7 +784,9 @@ def power_state_error_handler(e, node, power_state):
     if isinstance(e, exception.NoFreeConductorWorker):
         node.power_state = power_state
         node.target_power_state = states.NOSTATE
-        node.last_error = (_("No free conductor workers available"))
+        error = (_("No free conductor workers available"))
+        node_history_record(node, event=error, event_type=states.CONDUCTOR,
+                            error=True)
         node.save()
         LOG.warning("No free conductor workers available to perform "
                     "an action on node %(node)s, setting node's "
@@ -1032,7 +1077,7 @@ def fast_track_able(task):
               configuration is present, and no last_error is present for
               the node indicating that there was a recent failure.
     """
-    return (CONF.deploy.fast_track
+    return (utils.fast_track_enabled(task.node)
             # TODO(TheJulia): Network model aside, we should be able to
             # fast-track through initial sequence to complete deployment.
             # This needs to be validated.
@@ -1119,9 +1164,7 @@ def is_fast_track(task):
 
 def remove_agent_url(node):
     """Helper to remove the agent_url record."""
-    info = node.driver_internal_info
-    info.pop('agent_url', None)
-    node.driver_internal_info = info
+    node.del_driver_internal_info('agent_url')
 
 
 def _get_node_next_steps(task, step_type, skip_current_step=True):
@@ -1193,24 +1236,13 @@ def update_next_step_index(task, step_type):
     :param step_type: The type of steps to process: 'clean' or 'deploy'.
     :returns: Index of the next step.
     """
-    info = task.node.driver_internal_info
-    save_required = False
-
-    try:
-        skip_current_step = info.pop('skip_current_%s_step' % step_type)
-    except KeyError:
-        skip_current_step = True
+    skip_current_step = task.node.del_driver_internal_info(
+        'skip_current_%s_step' % step_type, True)
+    if step_type == 'clean':
+        task.node.del_driver_internal_info('cleaning_polling')
     else:
-        save_required = True
-
-    field = ('cleaning_polling' if step_type == 'clean'
-             else 'deployment_polling')
-    if info.pop(field, None) is not None:
-        save_required = True
-
-    if save_required:
-        task.node.driver_internal_info = info
-        task.node.save()
+        task.node.del_driver_internal_info('deployment_polling')
+    task.node.save()
 
     return _get_node_next_steps(task, step_type,
                                 skip_current_step=skip_current_step)
@@ -1226,13 +1258,11 @@ def add_secret_token(node, pregenerated=False):
                          the token is embedded into the configuration.
     """
     token = secrets.token_urlsafe()
-    i_info = node.driver_internal_info
-    i_info['agent_secret_token'] = token
+    node.set_driver_internal_info('agent_secret_token', token)
     if pregenerated:
-        i_info['agent_secret_token_pregenerated'] = True
+        node.set_driver_internal_info('agent_secret_token_pregenerated', True)
     else:
-        i_info.pop('agent_secret_token_pregenerated', None)
-    node.driver_internal_info = i_info
+        node.del_driver_internal_info('agent_secret_token_pregenerated')
 
 
 def is_agent_token_present(node):
@@ -1473,3 +1503,171 @@ def node_cache_boot_mode(task):
                  "for node %(node)s",
                  {'boot_mode': boot_mode, 'secure_boot': secure_boot,
                   'node': task.node.uuid})
+
+
+def node_change_boot_mode(task, target_boot_mode):
+    """Change boot mode to requested state for node
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param target_boot_mode: Any boot mode in :mod:`ironic.common.boot_modes`.
+    """
+    try:
+        current_boot_mode = task.driver.management.get_boot_mode(task)
+    except Exception as exc:
+        current_boot_mode = None
+        LOG.warning('Unexpected exception when trying to detect boot_mode '
+                    'while changing boot mode for node '
+                    '%(node)s. %(class)s: %(exc)s',
+                    {'node': task.node.uuid,
+                     'class': type(exc).__name__, 'exc': exc},
+                    exc_info=not isinstance(exc, exception.IronicException))
+
+    if (current_boot_mode is not None
+        and target_boot_mode == current_boot_mode):
+        LOG.info("Target boot mode '%(target)s', and current boot mode "
+                 "'%(current)s' are identical. No change being made "
+                 "for node %(node)s",
+                 {'target': target_boot_mode, 'current': current_boot_mode,
+                  'node': task.node.uuid})
+        return
+    try:
+        task.driver.management.set_boot_mode(task, mode=target_boot_mode)
+    except Exception as exc:
+        LOG.error('Unexpected exception when trying to change boot_mode '
+                  'to %(target)s for node %(node)s. %(class)s: %(exc)s',
+                  {'node': task.node.uuid, 'target': target_boot_mode,
+                   'class': type(exc).__name__, 'exc': exc},
+                  exc_info=not isinstance(exc, exception.IronicException))
+        task.node.last_error = (
+            "Failed to change boot mode to '%(target)s'. "
+            "Error: %(err)s" % {'target': target_boot_mode, 'err': exc})
+        task.node.save()
+    else:
+        LOG.info("Changed boot_mode to %(mode)s for node %(node)s",
+                 {'mode': target_boot_mode, 'node': task.node.uuid})
+        task.node.boot_mode = target_boot_mode
+        task.node.save()
+
+
+def node_change_secure_boot(task, secure_boot_target):
+    """Change secure_boot state to requested state for node
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param secure_boot_target: Target secure_boot state
+                                     OneOf(True => on, False => off)
+    :type secure_boot_target: boolean
+    """
+    try:
+        secure_boot_current = task.driver.management.get_secure_boot_state(
+            task)
+    except Exception as exc:
+        secure_boot_current = None
+        LOG.warning('Unexpected exception when trying to detect secure_boot '
+                    'state while changing secure_boot for node '
+                    '%(node)s. %(class)s: %(exc)s',
+                    {'node': task.node.uuid,
+                     'class': type(exc).__name__, 'exc': exc},
+                    exc_info=not isinstance(exc, exception.IronicException))
+
+    if (secure_boot_current is not None
+        and secure_boot_target == secure_boot_current):
+        LOG.info("Target secure_boot state '%(target)s', and current "
+                 "secure_boot state '%(current)s' are identical. "
+                 "No change being made for node %(node)s",
+                 {'target': secure_boot_target,
+                  'current': secure_boot_current,
+                  'node': task.node.uuid})
+        return
+    try:
+        task.driver.management.set_secure_boot_state(task, secure_boot_target)
+    except Exception as exc:
+        LOG.error('Unexpected exception when trying to change secure_boot '
+                  'to %(target)s for node %(node)s. %(class)s: %(exc)s',
+                  {'node': task.node.uuid, 'target': secure_boot_target,
+                   'class': type(exc).__name__, 'exc': exc},
+                  exc_info=not isinstance(exc, exception.IronicException))
+        task.node.last_error = (
+            "Failed to change secure_boot state to '%(target)s'. "
+            "Error: %(err)s" % {'target': secure_boot_target, 'err': exc})
+        task.node.save()
+    else:
+        LOG.info("Changed secure_boot state to %(state)s for node %(node)s",
+                 {'state': secure_boot_target, 'node': task.node.uuid})
+        task.node.secure_boot = secure_boot_target
+        task.node.save()
+
+
+def node_history_record(node, conductor=None, event=None,
+                        event_type=None, user=None,
+                        error=False):
+    """Records a node history record
+
+    Adds an entry to the node history table with the appropriate fields
+    populated to ensure consistent experience by also updating the
+    node ``last_error`` field. Please note the event is only recorded
+    if the ``[conductor]node_history_max_size`` parameter is set to a
+    value greater than ``0``.
+
+    :param node: A node object from a task object. Required.
+    :param conductor: The hostname of the conductor. If not specified
+                      this value is populated with the conductor FQDN.
+    :param event: The text to record to the node history table.
+                  If no value is supplied, the method silently returns
+                  to the caller.
+    :param event_type: The type activity where the event was encountered,
+                       either "provisioning", "monitoring", "cleaning",
+                       or whatever text the a driver author wishes to supply
+                       based upon the activity. The purpose is to help guide
+                       an API consumer/operator to have a better contextual
+                       understanding of what was going on *when* the "event"
+                       occured.
+    :param user: The user_id value which triggered the request,
+                 if available.
+    :param error: Boolean value, default false, to signify if the event
+                  is an error which should be recorded in the node
+                  ``last_error`` field.
+    :returns: None. No value is returned by this method.
+    """
+    if not event:
+        # No error has occured, apparently.
+        return
+    if error:
+        # When the task exits out or is saved, the event
+        # or error is saved, but that is outside of ceating an
+        # entry in the history table.
+        node.last_error = event
+    if not conductor:
+        conductor = CONF.host
+    if CONF.conductor.node_history:
+        # If the maximum number of entries is not set to zero,
+        # then we should record the entry.
+        # NOTE(TheJulia): DB API automatically adds in a uuid.
+        # TODO(TheJulia): At some point, we should allow custom severity.
+        node_history.NodeHistory(
+            node_id=node.id,
+            conductor=CONF.host,
+            user=user,
+            severity=error and "ERROR" or "INFO",
+            event=event,
+            event_type=event_type or "UNKNOWN").create()
+
+
+def update_image_type(context, node):
+    """Updates is_whole_disk_image and image_type based on the node data.
+
+    :param context: Request context.
+    :param node: Node object.
+    :return: True if any changes have been done, else False.
+    """
+    iwdi = images.is_whole_disk_image(context, node.instance_info)
+    if iwdi is None:
+        return False
+
+    node.set_driver_internal_info('is_whole_disk_image', iwdi)
+    # We need to gradually phase out is_whole_disk_image in favour of
+    # image_type, so make sure to set it as well. The primary use case is to
+    # cache information detected from Glance or the presence of kernel/ramdisk.
+    node.set_instance_info(
+        'image_type',
+        images.IMAGE_TYPE_WHOLE_DISK if iwdi else images.IMAGE_TYPE_PARTITION)
+    return True
