@@ -108,8 +108,14 @@ def parse_driver_info(node):
     if not parsed.scheme or not parsed.authority:
         address = 'https://%s' % address
         parsed = rfc3986.uri_reference(address)
-    # TODO(vdrok): Workaround this check, in py3 we need to use validator class
-    if not parsed.is_valid(require_scheme=True, require_authority=True):
+    validator = rfc3986.validators.Validator().require_presence_of(
+        'scheme', 'host',
+    ).check_validity_of(
+        'scheme', 'userinfo', 'host', 'path', 'query', 'fragment',
+    )
+    try:
+        validator.validate(parsed)
+    except rfc3986.exceptions.RFC3986Exception:
         raise exception.InvalidParameterValue(
             _('Invalid Redfish address %(address)s set in '
               'driver_info/redfish_address on node %(node)s') %
@@ -402,7 +408,7 @@ def get_enabled_macs(task, system):
     :param task: a TaskManager instance containing the node to act on.
     :param system: a Redfish System object
     :returns: a dictionary containing MAC addresses of enabled interfaces
-        in a {'mac': 'state'} format
+        in a {'mac': <state>} format, where <state> is a sushy constant
     """
 
     enabled_macs = {}
@@ -419,11 +425,11 @@ def get_enabled_macs(task, system):
                             "reported", {'node': task.node.uuid})
                 continue
             enabled_macs[nic_mac] = nic_state
-        if enabled_macs:
-            return enabled_macs
 
-    LOG.debug("No ethernet interface information is available "
-              "for node %(node)s", {'node': task.node.uuid})
+    if not enabled_macs:
+        LOG.debug("No ethernet interface information is available "
+                  "for node %(node)s", {'node': task.node.uuid})
+    return enabled_macs
 
 
 def wait_until_get_system_ready(node):
